from .automation import Automation
from .exceptions import AIHeroException


class TagEntireImages(Automation):
    def predict(self, image_url=None):
        if image_url is None or image_url.strip() == "":
            raise AIHeroException("image_url cannot be null or empty.")
        return super().infer("predict", {"image": image_url})



from aihero.exceptions import AIHeroException
from .api import Api
from .exceptions import AIHeroException
from .automation import Automation
from .detect_sentiment import DetectSentiment
from .tag_short_text import TagShortText
from .tag_entire_images import TagEntireImages
from .user_recommendation import UserRecommendation
from .item_recommendation import ItemRecommendation


class Client:
    """Main sync client class to talk to AI Hero"""

    automation_classes = {
        "detect_sentiment": DetectSentiment,
        "tag_short_text": TagShortText,
        "tag_entire_images": TagEntireImages,
        "item_recommendation": ItemRecommendation,
        "user_recommendation": UserRecommendation,
    }

    def __init__(self, api_key=None, server_url=None):
        self._api = Api(api_key=api_key, server_url=server_url)

    def ping(self):
        return self._api.get(
            self._api.ping_endpoint(),
            error_msg=f"Could not connect to the server {self._api.server_url}",
        )

    def get_automation(self, automation_id, api_key):
        automation = Automation(
            automation_id=automation_id,
            api=Api(api_key=api_key, server_url=self._api.server_url),
        )
        automation_definition = automation.get_definition()
        automation_type = automation_definition["type"]
        if automation_type not in Client.automation_classes:
            raise AIHeroException(
                f"Unsupported automation type for the API: {automation_type}"
            )
        automation_api = Api(api_key=api_key, server_url=self._api.server_url)
        automation = Client.automation_classes[automation_type](
            automation_id=automation_id, api=automation_api
        )
        return automation



from .automation import Automation
from .exceptions import AIHeroException


class DetectSentiment(Automation):
    def predict(self, text):
        if text is None or text.strip() == "":
            raise AIHeroException("text cannot be null or empty.")
        return super().infer("predict", {"text": text})



from .automation import Automation
from .exceptions import AIHeroException


class ItemRecommendation(Automation):
    def get_recommendations(self, item_id):
        if item_id is None or item_id.strip() == "":
            raise AIHeroException("item_id cannot be null or empty.")
        return super().infer("get_recommendations", {"thing_id": item_id})



import platform
import httpx
import urllib.parse

from .exceptions import AIHeroException

PRODUCTION_URL = "https://api.aihero.studio"


class Api:
    """Abstraction for http operations"""

    def __init__(self, api_key=None, server_url=None):
        self._headers = {
            "Content-Type": "application/json",
        }
        if api_key is None or api_key.strip() == "":
            self._api_key = None  # Only works for ping for now.
        else:
            self._api_key = api_key
            self._authorization = f"Bearer {self._api_key}"
            self._headers["Authorization"] = self._authorization

        if server_url.endswith("/"):
            server_url = server_url[:-1]
        self.server_url = server_url or PRODUCTION_URL

    def ping_endpoint(self):
        return f"{self.server_url}/ping"

    def endpoint(self, *args):
        return f"{self.server_url}/api/{'/'.join(args)}"

    def get(self, path, error_msg=None, network_errors=None):
        with httpx.Client() as client:
            response = client.get(path, headers=self._headers)
        try:
            response.raise_for_status()
        except httpx.HTTPStatusError as exc:
            if network_errors and exc.response.status_code in network_errors:
                raise AIHeroException(
                    f"{network_errors[exc.response.status_code]}",
                    status_code=exc.response.status_code,
                )
            elif exc.response.status_code:
                raise AIHeroException(error_msg, status_code=exc.response.status_code)
            else:
                raise AIHeroException(error_msg)
        return response.json()

    def post(self, path, obj, error_msg=None, network_errors=None):
        with httpx.Client() as client:
            response = client.post(path, json=obj, headers=self._headers)
        try:
            response.raise_for_status()
        except httpx.HTTPStatusError as exc:
            if network_errors and exc.response.status_code in network_errors:
                raise AIHeroException(
                    f"{network_errors[exc.response.status_code]}",
                    status_code=exc.response.status_code,
                )
            elif exc.response.status_code:
                raise AIHeroException(error_msg, status_code=exc.response.status_code)
            else:
                raise AIHeroException(error_msg)
        return response.json()



from .exceptions import AIHeroException


class Automation:
    """Abstraction for automation operations"""

    def __init__(self, automation_id, api):
        if automation_id is None or automation_id.strip() == "":
            raise AIHeroException("automation_id cannot be null or empty.")
        self._api = api
        self._automation_id = automation_id

    def get_definition(self):
        return self._api.get(
            self._api.endpoint("automations", self._automation_id, "definition"),
            error_msg="Couldn't get the definition.",
            network_errors={
                403: f"Could not connect to automation {self._automation_id}. Please check the API key."
            },
        )

    def infer(self, task, obj):
        return self._api.post(
            self._api.endpoint("automations", self._automation_id, "inferences", task),
            obj,
            error_msg=f"Unknown error in {task}.",
            network_errors={
                400: f"Please check the data uploaded.",
                403: f"Could not connect to automation {self._automation_id}. Please check the API key.",
            },
        )



from .automation import Automation
from .exceptions import AIHeroException


class UserRecommendation(Automation):
    def get_recommendations(self, person_id):
        if person_id is None or person_id.strip() == "":
            raise AIHeroException("person_id cannot be null or empty.")
        return super().infer("get_recommendations", {"thing_id": person_id})
